/*************************************************************************

   File:		P2AT.h

   Author(s):		  Ioannis Rekleitis

   Created:		 26/8/09

   Last Revision:	$Date:  $

   Description:	Interface of the Robot class.  Connects to the Player/Stage
                multi-robot simulator.

 ***********************************************************************/

#ifndef P2AT_H
#define P2AT_H
using namespace std;

#include <string>
#include "Pose2D.h"
#include <vector>
#include <libplayerinterface/player.h>
// #include <libplayercore/player.h>
#include <libplayerc++/playerc++.h>
#include <libplayerc++/utility.h>

using namespace PlayerCc;

class P2AT
{
 public:
  
  P2AT(const char * host, const int port, const int index); //constructor
  virtual ~P2AT(){} //destructor
  int getID () { return Id; }


  Pose2D getPose();
  void setPose(Pose2D nP);

  // Motion commands
  void move(double tVel, double aVel);
  void setWheelSpeed(double lwVel, double rwVel);
  void translate(double t);
  void absRotate(double a);
  void relRotate(double a);
  void stop();

  // Sensing commands
  vector<double> getLaser();
  vector<double> getSonar();



 private:
  int Id;
  string Name;

  PlayerClient    rr;
  Position2dProxy pp;
  LaserProxy      lp;
  SonarProxy      sp;
  SimulationProxy simp;

};

#endif	// P2AT_H
