/////////////////////////////////////////////////////////////////////////////
//   File:                Point2D.h
//   Author(s):           Luiza Solomon, Ioannis Rekleitis
//
//   Created:             11 Jun 2003
//
//   Last Revision:       August 1st, 2003
//
//   Description:         Supplies the necessary point functionality to
//                        replace LEDA library calls. 
//
//   Copyright (c) 2003 by Ioannis Rekleitis, Centre for Intelligent Machines,
//   McGill University, Montreal, QC.  Please see the copyright notice
//   included in this distribution for full details.
//
///////////////////////////////////////////////////////////////////////

#ifndef POINT2D_H
#define POINT2D_H

#include <iostream>
#include <math.h>
using namespace std;

//////////////////////////////////////////////////////////////////
//Declaring the Point2D class

class Point2D
{
 public:

    Point2D() { x = 0; y = 0; }
    Point2D(double xcoord, double ycoord) { x = xcoord; y = ycoord; }
    ~Point2D() {}
    Point2D(const Point2D &rhs); // copy constructor

    bool operator==(const Point2D &rhs) const;
    bool operator!=(const Point2D &rhs) const;
    Point2D& operator=(const Point2D &rhs); 
    friend ostream &operator<<(ostream &os, const Point2D &rline);
    void Print(ostream &os);
    friend istream &operator>>(istream &is, Point2D &rline);

    double xcoord() const { return x; }
    double ycoord() const { return y; }

/*     void Setxy(double xcoord, double ycoord) {x = xcoord; y = ycoord;} */
/*     void Setx(double xcoord) {x = xcoord;} */
/*     void Sety(double ycoord) {y = ycoord;} */

    // returns the distance between current point and point q
    double distance(const Point2D q) const;
    // returns manhatten distance
    double mdistance(const Point2D q) const;

    // computes the orientation of points a, b, and c 
    int orientation(const Point2D q, const Point2D r) const;

    // returns the point rotated about origin by angle fi
    Point2D rotate(const Point2D &origin, double fi) const;

    // returns rotate(point(0, 0), fi)
    Point2D rotate(double fi) const;

    // returns the point rotated about p by an angle of 90 degrees
    Point2D rotate90(const Point2D &p) const;

    // returns rotate90(point(0, 0))
    Point2D rotate90() const;

 private:

    double x, y;   // x, y coordinates of a point
};

#endif

////////////////////////////////////////////////////////////////////////
// EOF
