/////////////////////////////////////////////////////////////////////////////
//   File:                Pose2D.h
//   Author(s):           Ioannis Rekleitis
//
//   Created:             16 Sep 2003
//
//   Last Revision:       Sept. 16, 2003
//
//   Description:         Supplies 2D robot Pose functionality. The angles 
//                        are ALWAYS in radians. The only time they are in 
//                        degrees is when the are outputed or inputed. 
//                        Moreover, the angles are always stored in the 
//                        interval [0,2pi).   
//
//   Copyright (c) 2003 by Ioannis Rekleitis, Carnegie Mellon University.
//   Please see the copyright notice included in this distribution for 
//   full details.
//
///////////////////////////////////////////////////////////////////////

#ifndef POSE2D_H
#  define POSE2D_H

#include <iostream>
#include <math.h>
#include "Point2D.h"
using namespace std;

//////////////////////////////////////////////////////////////////
//Declaring the Pose2D class



class Pose2D 
{
 public:

    Pose2D() { x = 0.0; y = 0.0; theta=0.0;}
    Pose2D(double xcoord, double ycoord, double thetaV) 
      { x = xcoord; y = ycoord; theta=theta02p(thetaV); }
    ~Pose2D() {}
    Pose2D(const Pose2D &rhs); // copy constructor

    bool operator==(const Pose2D &rhs) const;
    bool operator!=(const Pose2D &rhs) const;
    Pose2D& operator=(const Pose2D &rhs); 
    friend ostream &operator<<(ostream &os, const Pose2D &rline);
    friend istream &operator>>(istream &is, Pose2D &rline);

    double xcoord() const { return x; }
    double ycoord() const { return y; }
    double orientation()  const { return theta; }
    Point2D Position() const { return Point2D(x,y); }

/*     void SetPose2D(double xcoord, double ycoord, double thetaV)  */
/*       { x = xcoord; y = ycoord; theta=(thetaV); } */
/*     void Setxy(double xcoord, double ycoord) {x = xcoord; y = ycoord} */
/*     void Setx(double xcoord) {x = xcoord;} */
/*     void Sety(double ycoord) {y = ycoord;} */
/*     void Settheta(double thetaV) {theta = theta02p(thetaV);} */


    // returns the distance between current pose and pose q
    double distance(const Pose2D q) const;
    double angleDiffDeg(double B) ;
    double angleDiff(Pose2D B);

    double theta0360(double t);
    double theta180(double t);
    double theta02p(double t);
    double thetapp(double t);
    double r2d(const double r) const { return (r*Rad2Deg); }
    double d2r(const double d) const { return (d*Deg2Rad); }
    //protected:

 private:
    static const double mPI;     // 3.14159265;
    static const double Deg2Rad; // 0.0174532925;
    static const double Rad2Deg; // 57.2957795786

    double x, y, theta;   // x, y coord. of a robot theta orientation in rad


};

#endif

////////////////////////////////////////////////////////////////////////
// EOF
