#include <iostream>
#include <libplayerc++/playerc++.h>

int DoDisturbance = 0;
#define PI 3.14159265

int
main(int argc, char *argv[])
{
  using namespace PlayerCc;

  FILE *myFile = fopen("myfile.txt","w+");
  if (myFile == NULL) {
  	std::cout << "File not open!!!" << std::endl;
  	return 0;
  }
  	
  PlayerClient    robot("localhost");
  SonarProxy      sp(&robot,0);
  LaserProxy      lp(&robot,0);
  Position2dProxy pp(&robot,0);
 
  double KpInner = 0.4;
  double KdInner = 0.0;
  double KpOuter = 1.0;
  
  // Descrption of the line
  double p1x[] = { -7,  5,  6, -6, -7};
  double p2x[] = {  5,  6, -6,  7,  9};
  double p1y[] = { -5, -5,  6,  0, -7};
  double p2y[] = { -5,  6,  0, -7, -7};
  
  int nSegments = 5;
  int ActiveSegment = 0;
  double OldErrorAngle = 0.0;
  
  bool RandomNoiseMotor  = false;
  bool RandomNoiseSensor = false;
  bool WallFollowing     = false;
  
  double delay[20];
  int DelayDuration = 0;
  int i;
  for (i = 0; i < DelayDuration; i++) {
  	delay[i] = 0.0;
  }
  
  double TargetDist = 1.0;
  for(;;)
  {
    double turnrate, speed, ThetaC;
    double x1,x2,y1,y2;
    double LineAngle, ThetaR,Dist;
    speed = 0.2;

    // read from the proxies
    robot.Read();
	
	if (WallFollowing) {
		// Find two points corresponding to the wall
		int MaxCount = lp.GetCount();
		// Read all the data to find the minimum
		double MinDist = 100.0;
		int MinIndex = -1;
		for (i=0; i< MaxCount; i++) {
			if (lp[i] < MinDist) {
				MinIndex = i;
				MinDist = lp[i];
			}
		}
		std::cout << lp.GetBearing(MinIndex) << " " << MinDist << std::endl;
		ThetaR = -lp.GetBearing(MinIndex) + PI*0.5;
		Dist = MinDist;
	}
	else {
		x2 = p2x[ActiveSegment];
		x1 = p1x[ActiveSegment];
		y2 = p2y[ActiveSegment];
		y1 = p1y[ActiveSegment];
		LineAngle = atan2(y2 - y1, x2 - x1);
		ThetaR    = pp.GetYaw() - LineAngle;
	    Dist = ((x2-x1)*(y1-pp.GetYPos())-(x1-pp.GetXPos())*(y2-y1))/sqrt(pow(x2-x1,2) + pow(y2-y1,2));
	}
	
	
	if (RandomNoiseSensor) {
		double MaxVal = 1.0;
		double Noise = MaxVal*2.0*double(rand())/double(RAND_MAX)-MaxVal;
		ThetaR += Noise;
	}
	
	
	if (ThetaR > PI) {
		ThetaR -= 2*PI;
		std::cout << "a" << std::endl;
	}
	else if (ThetaR < -PI) {
		ThetaR += 2*PI;
		std::cout << "b" << std::endl;
	}	
	// Outer Loop
	
	ThetaC   = KpOuter * (Dist-TargetDist);

	double AngleLimit = 0.5*PI;
	if (ThetaC > AngleLimit) {
		ThetaC = AngleLimit;
	}
	else if (ThetaC < -AngleLimit) {
		ThetaC = -AngleLimit;
	}
	
	// Computing the Derror
	
	double ErrorAngle = (ThetaC - ThetaR);
	double Derror = (ErrorAngle - OldErrorAngle)*0.1;
	OldErrorAngle = ErrorAngle;
		
	turnrate = KpInner * ErrorAngle + KdInner*Derror;
	
	if (RandomNoiseMotor) {
		double MaxVal = 0.4;
		double Noise = MaxVal*2.0*double(rand())/double(RAND_MAX)-MaxVal;
		turnrate = turnrate + Noise;
	}
	
	if (DelayDuration > 0) {
		double out = delay[0];
		for (i = 0; i < DelayDuration; i++) {
    		delay[i] = delay[i+1];
		}
		delay[DelayDuration] = turnrate;
		turnrate = out;
	}
	
	printf("LineAngle: %.2f ThetaC: %.2f ThetaR: %.2f turnrate: %.4f dist: %.2f\n",
	        LineAngle,ThetaC ,ThetaR,turnrate,Dist);
	fprintf(myFile," %.2f, %.2f, %.2f\n",pp.GetXPos(), pp.GetYPos(), Dist);
    // command the motors
    pp.SetSpeed(speed, turnrate);
    
    // Check to see if we need to change segment
    if (sqrt( pow(x2 - pp.GetXPos(),2) + pow(y2 - pp.GetYPos(),2)) < 0.9) {
    	std::cout << "***** Switch Line" << std::endl;
    	ActiveSegment++;
    	if (ActiveSegment >= nSegments) {
    		return 0;
    		fclose(myFile);
    	}
    }
  }
}

