#include <libplayerc++/playerc++.h>

int main(int argc, char *argv[]) {
	using namespace PlayerCc;
	// set up proxies
	PlayerClient robot("localhost", 6665);
	Position2dProxy pp(&robot, 0);
	SimulationProxy simp(&robot, 0);
	// declare constants
	const double DT = 0.1;
	const double KP = 0.1;
	const double KD = 0.15;
	const int NUMBER_OF_POINTS = 6;
	const int POINTS[NUMBER_OF_POINTS][2] = {{-7, -5}, {5, -5}, {6, 6}, {-6, 0}, {7, -7}, {9, -7}};
	const double SPEED = 0.2;
	// declare variables
	int i = 0, j = 0;
	double error, constant, closestPointX, closestPointY, derivative, distanceToPoint, previousError = 0, totalError = 0, turnRate;
	// set intial position
  	simp.SetPose2d (const_cast<char*>("robot1"), -7, -4.9, 0);
	std::cout << "KP = " <<  KP << " KD = " << KD << std::endl;
	for(;;) {
		robot.Read();
		// calculate closest point on line
		constant = ((pp.GetXPos()-POINTS[i][0])*(POINTS[i+1][0]-POINTS[i][0])+(pp.GetYPos()-POINTS[i][1])*(POINTS[i+1][1]-POINTS[i][1]))/(pow((POINTS[i+1][0]-POINTS[i][0]), 2)+pow((POINTS[i+1][1]-POINTS[i][1]), 2)); 
		closestPointX = POINTS[i][0]+constant*(POINTS[i+1][0]-POINTS[i][0]);
		closestPointY = POINTS[i][1]+constant*(POINTS[i+1][1]-POINTS[i][1]);
		// calculate error
		error = sqrt(pow(closestPointX-pp.GetXPos(), 2)+pow(closestPointY-pp.GetYPos(), 2));
		totalError += error;
		// calculate distance to last point of the line		
		distanceToPoint = sqrt(pow((pp.GetXPos()-POINTS[i+1][0]), 2) + pow((pp.GetYPos()-POINTS[i+1][1]), 2));
		if (distanceToPoint < 0.5) { i++; }
		// if last point reached then stop
		if (i == (NUMBER_OF_POINTS-1)) { break; }
		// PID
		derivative = (error-previousError) / DT;
		turnRate = KP * error + KD * derivative;
		previousError = error;
		double x, y, a;
	   	simp.GetPose2d(const_cast<char*>("robot1"), x, y, a);
		std::cout << x << ", " << y << ", " << turnRate << ", " << error << std::endl;
		pp.SetSpeed(SPEED, turnRate);
		sleep(DT);
		j++;
	}
	std::cout << "Average error: " <<  totalError/j << std::endl;
}
